//redThermoKontroll
//redFrik 2010 gnu gpl v2
//mirf and spi libraries by stefan engelke

//outgoing wireless protocol: slavindex, a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, dig, 0

#include <util/delay.h>
#include "nRF24L01.h"
#include "spi.c"
#include "mirf.c"

#define SLAVEINDEX 1								//0= redUniform, 1= redThermoKontroll etc.
#define FOSC 8000000
#define master_PAYLOAD 16
#define UPDATERATE 40

void setChannel(uint8_t index);
uint8_t readAdc();
uint8_t outgoing[master_PAYLOAD];					//data to send to master
uint8_t outCopy[master_PAYLOAD];					//temp storage
uint8_t changed;									//flag
uint8_t i, j= 0;

int main(void) {
	
	//--set pull-ups for switches
	PORTC= _BV(PC2) | _BV(PC1) | _BV(PC0);
	
	//--set outputs
	DDRD = _BV(DDD7) | _BV(DDD6) | _BV(DDD5) | _BV(DDD4) | _BV(DDD3);
	
	//--adc init
	ADMUX |= (1<<REFS0);							//voltage reference
	ADCSRA |= ((1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0));	//prescaler 64
	ADCSRA |= (1<<ADEN);							//enable adc
	
	//--mirf init
	mirf_init();
	_delay_ms(50);
	sei();
	mirf_config();
	uint8_t buf[5];
	buf[0]= 10;
	buf[1]= 11;
	buf[2]= 12;
	buf[3]= 13;
	buf[4]= SLAVEINDEX;								//device address
	mirf_set_RADDR(buf);							//rx address
	buf[0]= 20;
	buf[1]= 21;
	buf[2]= 22;
	buf[3]= 23;
	buf[4]= 24;
	mirf_set_TADDR(buf);							//tx address
	
	//--led startup routine
	for(i= 1; i<5; i++) {
		PORTD |= _BV(DDD7);							//set red led on
		_delay_ms(i*10);
		PORTD &= ~_BV(DDD7);						//set red led off
		_delay_ms(i*10);
	}
	
	outgoing[0]= SLAVEINDEX;						//device id never changes
	outgoing[15]= 0;								//zero padding
	
	//--main loop
	while(1) {
		
		changed= 0;
		
		//--read analog inputs
		for(i= 1; i<14; i++) {
			setChannel(i-1);
			outgoing[i]= readAdc();
			if(outCopy[i]!=outgoing[i]) {
				outCopy[i]= outgoing[i];
				changed= 1;
			}
		}
		
		//--read digital inputs
		outgoing[14]= PINC&0b00011111;
		if(outCopy[14]!=outgoing[14]) {
			outCopy[14]= outgoing[14];
			changed= 1;
		}
		
		//--send wireless
		if(changed) {
			mirf_send(outgoing, master_PAYLOAD);
			_delay_ms(UPDATERATE);
			if(j%2==0) {
				 PORTD &= ~_BV(DDD7);				//set red led off
			} else {
				 PORTD |= _BV(DDD7);				//set red led on
			}
			j++;
		}
	}
	
	return 0;
}
void setChannel(uint8_t index) {
	switch(index) {
		case 0:
			PORTD &= ~_BV(DDD3);					//low
			PORTD &= ~_BV(DDD4);					//low
			PORTD &= ~_BV(DDD5);					//low
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 1:
			PORTD |= _BV(DDD3);						//high
			PORTD &= ~_BV(DDD4);					//low
			PORTD &= ~_BV(DDD5);					//low
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 2:
			PORTD &= ~_BV(DDD3);					//low
			PORTD |= _BV(DDD4);						//high
			PORTD &= ~_BV(DDD5);					//low
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 3:
			PORTD |= _BV(DDD3);						//high
			PORTD |= _BV(DDD4);						//high
			PORTD &= ~_BV(DDD5);					//low
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 4:
			PORTD &= ~_BV(DDD3);					//low
			PORTD &= ~_BV(DDD4);					//low
			PORTD |= _BV(DDD5);						//high
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 5:
			PORTD |= _BV(DDD3);						//high
			PORTD &= ~_BV(DDD4);					//low
			PORTD |= _BV(DDD5);						//high
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 6:
			PORTD &= ~_BV(DDD3);					//low
			PORTD |= _BV(DDD4);						//high
			PORTD |= _BV(DDD5);						//high
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 7:
			PORTD |= _BV(DDD3);						//high
			PORTD |= _BV(DDD4);						//high
			PORTD |= _BV(DDD5);						//high
			PORTD &= ~_BV(DDD6);					//low
			break;
		case 8:
			PORTD &= ~_BV(DDD3);					//low
			PORTD &= ~_BV(DDD4);					//low
			PORTD &= ~_BV(DDD5);					//low
			PORTD |= _BV(DDD6);						//high
			break;
		case 9:
			PORTD |= _BV(DDD3);						//high
			PORTD &= ~_BV(DDD4);					//low
			PORTD &= ~_BV(DDD5);					//low
			PORTD |= _BV(DDD6);						//high
			break;
		case 10:
			PORTD &= ~_BV(DDD3);					//low
			PORTD |= _BV(DDD4);						//high
			PORTD &= ~_BV(DDD5);					//low
			PORTD |= _BV(DDD6);						//high
			break;
		case 11:
			PORTD |= _BV(DDD3);						//high
			PORTD |= _BV(DDD4);						//high
			PORTD &= ~_BV(DDD5);					//low
			PORTD |= _BV(DDD6);						//high
			break;
		case 12:
			PORTD &= ~_BV(DDD3);					//low
			PORTD &= ~_BV(DDD4);					//low
			PORTD |= _BV(DDD5);						//high
			PORTD |= _BV(DDD6);						//high
			break;
	}
			
}
uint8_t readAdc() {
	ADMUX &= ~0x0f;
	ADMUX |= 5;										//set channel
	ADCSRA |= (1<<ADSC);							//start conversion
	while(bit_is_set(ADCSRA, ADSC));				//wait until conversion finished
	return ADCW>>2;									//read 10bit value and scale down to 8bit
}
